@@ -34,27 +34,31 @@
 #include <linux/if_link.h>
 
 /** @cond SKIP */
-#define VXLAN_HAS_ID	(1<<0)
-#define VXLAN_HAS_GROUP	(1<<1)
-#define VXLAN_HAS_LINK	(1<<2)
-#define VXLAN_HAS_LOCAL	(1<<3)
-#define VXLAN_HAS_TTL	(1<<4)
-#define VXLAN_HAS_TOS	(1<<5)
-#define VXLAN_HAS_LEARNING	(1<<6)
-#define VXLAN_HAS_AGEING	(1<<7)
-#define VXLAN_HAS_LIMIT		(1<<8)
-#define VXLAN_HAS_PORT_RANGE	(1<<9)
-#define VXLAN_HAS_PROXY	(1<<10)
-#define VXLAN_HAS_RSC	(1<<11)
-#define VXLAN_HAS_L2MISS	(1<<12)
-#define VXLAN_HAS_L3MISS	(1<<13)
+#define VXLAN_ATTR_ID           (1<<0)
+#define VXLAN_ATTR_GROUP        (1<<1)
+#define VXLAN_ATTR_LINK         (1<<2)
+#define VXLAN_ATTR_LOCAL        (1<<3)
+#define VXLAN_ATTR_TTL          (1<<4)
+#define VXLAN_ATTR_TOS          (1<<5)
+#define VXLAN_ATTR_LEARNING     (1<<6)
+#define VXLAN_ATTR_AGEING       (1<<7)
+#define VXLAN_ATTR_LIMIT        (1<<8)
+#define VXLAN_ATTR_PORT_RANGE   (1<<9)
+#define VXLAN_ATTR_PROXY        (1<<10)
+#define VXLAN_ATTR_RSC          (1<<11)
+#define VXLAN_ATTR_L2MISS       (1<<12)
+#define VXLAN_ATTR_L3MISS       (1<<13)
+#define VXLAN_ATTR_GROUP6       (1<<14)
+#define VXLAN_ATTR_LOCAL6       (1<<15)
 
 struct vxlan_info
 {
 	uint32_t		vxi_id;
 	uint32_t		vxi_group;
+	struct in6_addr		vxi_group6;
 	uint32_t		vxi_link;
 	uint32_t		vxi_local;
+	struct in6_addr		vxi_local6;
 	uint8_t			vxi_ttl;
 	uint8_t			vxi_tos;
 	uint8_t			vxi_learning;
@@ -65,7 +69,7 @@ struct vxlan_info
 	uint8_t			vxi_rsc;
 	uint8_t			vxi_l2miss;
 	uint8_t			vxi_l3miss;
-	uint32_t		vxi_mask;
+	uint32_t		ce_mask;
 };
 
 /** @endcond */
@@ -73,8 +77,10 @@ struct vxlan_info
 static struct nla_policy vxlan_policy[IFLA_VXLAN_MAX+1] = {
 	[IFLA_VXLAN_ID]	= { .type = NLA_U32 },
 	[IFLA_VXLAN_GROUP] = { .minlen = sizeof(uint32_t) },
+	[IFLA_VXLAN_GROUP6] = { .minlen = sizeof(struct in6_addr) },
 	[IFLA_VXLAN_LINK] = { .type = NLA_U32 },
 	[IFLA_VXLAN_LOCAL] = { .minlen = sizeof(uint32_t) },
+	[IFLA_VXLAN_LOCAL6] = { .minlen = sizeof(struct in6_addr) },
 	[IFLA_VXLAN_TTL] = { .type = NLA_U8 },
 	[IFLA_VXLAN_TOS] = { .type = NLA_U8 },
 	[IFLA_VXLAN_LEARNING] = { .type = NLA_U8 },
@@ -91,10 +97,14 @@ static int vxlan_alloc(struct rtnl_link *link)
 {
 	struct vxlan_info *vxi;
 
-	if ((vxi = calloc(1, sizeof(*vxi))) == NULL)
-		return -NLE_NOMEM;
+	if (link->l_info)
+		memset(link->l_info, 0, sizeof(*vxi));
+	else {
+		if ((vxi = calloc(1, sizeof(*vxi))) == NULL)
+			return -NLE_NOMEM;
 
-	link->l_info = vxi;
+		link->l_info = vxi;
+	}
 
 	return 0;
 }
@@ -106,7 +116,7 @@ static int vxlan_parse(struct rtnl_link *link, struct nlattr *data,
 	struct vxlan_info *vxi;
 	int err;
 
-	NL_DBG(3, "Parsing VXLAN link info");
+	NL_DBG(3, "Parsing VXLAN link info\n");
 
 	if ((err = nla_parse_nested(tb, IFLA_VXLAN_MAX, data, vxlan_policy)) < 0)
 		goto errout;
@@ -118,75 +128,89 @@ static int vxlan_parse(struct rtnl_link *link, struct nlattr *data,
 
 	if (tb[IFLA_VXLAN_ID]) {
 		vxi->vxi_id = nla_get_u32(tb[IFLA_VXLAN_ID]);
-		vxi->vxi_mask |= VXLAN_HAS_ID;
+		vxi->ce_mask |= VXLAN_ATTR_ID;
+	}
+
+	if (tb[IFLA_VXLAN_GROUP6]) {
+		nla_memcpy(&vxi->vxi_group6, tb[IFLA_VXLAN_GROUP6],
+			   sizeof(vxi->vxi_group6));
+		vxi->ce_mask |= VXLAN_ATTR_GROUP6;
 	}
 
 	if (tb[IFLA_VXLAN_GROUP]) {
 		nla_memcpy(&vxi->vxi_group, tb[IFLA_VXLAN_GROUP],
 				   sizeof(vxi->vxi_group));
-		vxi->vxi_mask |= VXLAN_HAS_GROUP;
+		vxi->ce_mask |= VXLAN_ATTR_GROUP;
+		vxi->ce_mask &= ~VXLAN_ATTR_GROUP6;
 	}
 
 	if (tb[IFLA_VXLAN_LINK]) {
 		vxi->vxi_link = nla_get_u32(tb[IFLA_VXLAN_LINK]);
-		vxi->vxi_mask |= VXLAN_HAS_LINK;
+		vxi->ce_mask |= VXLAN_ATTR_LINK;
+	}
+
+	if (tb[IFLA_VXLAN_LOCAL6]) {
+		nla_memcpy(&vxi->vxi_local6, tb[IFLA_VXLAN_LOCAL6],
+			   sizeof(vxi->vxi_local6));
+		vxi->ce_mask |= VXLAN_ATTR_LOCAL6;
 	}
 
 	if (tb[IFLA_VXLAN_LOCAL]) {
 		nla_memcpy(&vxi->vxi_local, tb[IFLA_VXLAN_LOCAL],
 				   sizeof(vxi->vxi_local));
-		vxi->vxi_mask |= VXLAN_HAS_LOCAL;
+		vxi->ce_mask |= VXLAN_ATTR_LOCAL;
+		vxi->ce_mask &= ~VXLAN_ATTR_LOCAL6;
 	}
 
 	if (tb[IFLA_VXLAN_TTL]) {
 		vxi->vxi_ttl = nla_get_u8(tb[IFLA_VXLAN_TTL]);
-		vxi->vxi_mask |= VXLAN_HAS_TTL;
+		vxi->ce_mask |= VXLAN_ATTR_TTL;
 	}
 
 	if (tb[IFLA_VXLAN_TOS]) {
 		vxi->vxi_tos = nla_get_u8(tb[IFLA_VXLAN_TOS]);
-		vxi->vxi_mask |= VXLAN_HAS_TOS;
+		vxi->ce_mask |= VXLAN_ATTR_TOS;
 	}
 
 	if (tb[IFLA_VXLAN_LEARNING]) {
 		vxi->vxi_learning = nla_get_u8(tb[IFLA_VXLAN_LEARNING]);
-		vxi->vxi_mask |= VXLAN_HAS_LEARNING;
+		vxi->ce_mask |= VXLAN_ATTR_LEARNING;
 	}
 
 	if (tb[IFLA_VXLAN_AGEING]) {
 		vxi->vxi_ageing = nla_get_u32(tb[IFLA_VXLAN_AGEING]);
-		vxi->vxi_mask |= VXLAN_HAS_AGEING;
+		vxi->ce_mask |= VXLAN_ATTR_AGEING;
 	}
 
 	if (tb[IFLA_VXLAN_LIMIT]) {
 		vxi->vxi_limit = nla_get_u32(tb[IFLA_VXLAN_LIMIT]);
-		vxi->vxi_mask |= VXLAN_HAS_LIMIT;
+		vxi->ce_mask |= VXLAN_ATTR_LIMIT;
 	}
 
 	if (tb[IFLA_VXLAN_PORT_RANGE]) {
 		nla_memcpy(&vxi->vxi_port_range, tb[IFLA_VXLAN_PORT_RANGE],
 				   sizeof(vxi->vxi_port_range));
-		vxi->vxi_mask |= VXLAN_HAS_PORT_RANGE;
+		vxi->ce_mask |= VXLAN_ATTR_PORT_RANGE;
 	}
 
 	if (tb[IFLA_VXLAN_PROXY]) {
 		vxi->vxi_proxy = nla_get_u8(tb[IFLA_VXLAN_PROXY]);
-		vxi->vxi_mask |= VXLAN_HAS_PROXY;
+		vxi->ce_mask |= VXLAN_ATTR_PROXY;
 	}
 
 	if (tb[IFLA_VXLAN_RSC]) {
 		vxi->vxi_rsc = nla_get_u8(tb[IFLA_VXLAN_RSC]);
-		vxi->vxi_mask |= VXLAN_HAS_RSC;
+		vxi->ce_mask |= VXLAN_ATTR_RSC;
 	}
 
 	if (tb[IFLA_VXLAN_L2MISS]) {
 		vxi->vxi_l2miss = nla_get_u8(tb[IFLA_VXLAN_L2MISS]);
-		vxi->vxi_mask |= VXLAN_HAS_L2MISS;
+		vxi->ce_mask |= VXLAN_ATTR_L2MISS;
 	}
 
 	if (tb[IFLA_VXLAN_L3MISS]) {
 		vxi->vxi_l3miss = nla_get_u8(tb[IFLA_VXLAN_L3MISS]);
-		vxi->vxi_mask |= VXLAN_HAS_L3MISS;
+		vxi->ce_mask |= VXLAN_ATTR_L3MISS;
 	}
 
 	err = 0;
@@ -213,36 +237,55 @@ static void vxlan_dump_line(struct rtnl_link *link, struct nl_dump_params *p)
 static void vxlan_dump_details(struct rtnl_link *link, struct nl_dump_params *p)
 {
 	struct vxlan_info *vxi = link->l_info;
-	char *name, addr[INET_ADDRSTRLEN];
+	char *name, addr[INET6_ADDRSTRLEN];
+	struct rtnl_link *parent;
 
 	nl_dump_line(p, "    vxlan-id %u\n", vxi->vxi_id);
 
-	if (vxi->vxi_mask & VXLAN_HAS_GROUP) {
+	if (vxi->ce_mask & VXLAN_ATTR_GROUP) {
 		nl_dump(p, "      group ");
-		if(inet_ntop(AF_INET, &vxi->vxi_group, addr, sizeof(addr)))
+		if (inet_ntop(AF_INET, &vxi->vxi_group, addr, sizeof(addr)))
 			nl_dump_line(p, "%s\n", addr);
 		else
 			nl_dump_line(p, "%#x\n", ntohs(vxi->vxi_group));
+	} else if (vxi->ce_mask & VXLAN_ATTR_GROUP6) {
+		nl_dump(p, "      group ");
+		if (inet_ntop(AF_INET6, &vxi->vxi_group6, addr, sizeof(addr)))
+			nl_dump_line(p, "%s\n", addr);
+		else
+			nl_dump_line(p, "%#x\n", vxi->vxi_group6);
 	}
 
-	if (vxi->vxi_mask & VXLAN_HAS_LINK) {
+	if (vxi->ce_mask & VXLAN_ATTR_LINK) {
 		nl_dump(p, "      link ");
-		name = rtnl_link_get_name(link);
+
+		name = NULL;
+		parent = link_lookup(link->ce_cache, vxi->vxi_link);
+		if (parent)
+			name = rtnl_link_get_name(parent);
+
 		if (name)
 			nl_dump_line(p, "%s\n", name);
 		else
 			nl_dump_line(p, "%u\n", vxi->vxi_link);
 	}
 
-	if (vxi->vxi_mask & VXLAN_HAS_LOCAL) {
+	if (vxi->ce_mask & VXLAN_ATTR_LOCAL) {
 		nl_dump(p, "      local ");
-		if(inet_ntop(AF_INET, &vxi->vxi_local, addr, sizeof(addr)))
+		if (inet_ntop(AF_INET, &vxi->vxi_local, addr, sizeof(addr)))
 			nl_dump_line(p, "%s\n", addr);
 		else
 			nl_dump_line(p, "%#x\n", ntohs(vxi->vxi_local));
+	} else if (vxi->ce_mask & VXLAN_ATTR_LOCAL6) {
+		nl_dump(p, "      local ");
+		if (inet_ntop(AF_INET6, &vxi->vxi_local6, addr, sizeof(addr)))
+			nl_dump_line(p, "%s\n", addr);
+		else
+			nl_dump_line(p, "%#x\n", vxi->vxi_local6);
 	}
 
-	if (vxi->vxi_mask & VXLAN_HAS_TTL) {
+
+	if (vxi->ce_mask & VXLAN_ATTR_TTL) {
 		nl_dump(p, "      ttl ");
 		if(vxi->vxi_ttl)
 			nl_dump_line(p, "%u\n", vxi->vxi_ttl);
@@ -250,7 +293,7 @@ static void vxlan_dump_details(struct rtnl_link *link, struct nl_dump_params *p)
 			nl_dump_line(p, "inherit\n");
 	}
 
-	if (vxi->vxi_mask & VXLAN_HAS_TOS) {
+	if (vxi->ce_mask & VXLAN_ATTR_TOS) {
 		nl_dump(p, "      tos ");
 		if (vxi->vxi_tos == 1)
 			nl_dump_line(p, "inherit\n", vxi->vxi_tos);
@@ -258,7 +301,7 @@ static void vxlan_dump_details(struct rtnl_link *link, struct nl_dump_params *p)
 			nl_dump_line(p, "%#x\n", vxi->vxi_tos);
 	}
 
-	if (vxi->vxi_mask & VXLAN_HAS_LEARNING) {
+	if (vxi->ce_mask & VXLAN_ATTR_LEARNING) {
 		nl_dump(p, "      learning ");
 		if (vxi->vxi_learning)
 			nl_dump_line(p, "enabled (%#x)\n", vxi->vxi_learning);
@@ -266,7 +309,7 @@ static void vxlan_dump_details(struct rtnl_link *link, struct nl_dump_params *p)
 			nl_dump_line(p, "disabled\n");
 	}
 
-	if (vxi->vxi_mask & VXLAN_HAS_AGEING) {
+	if (vxi->ce_mask & VXLAN_ATTR_AGEING) {
 		nl_dump(p, "      ageing ");
 		if (vxi->vxi_ageing)
 			nl_dump_line(p, "%u seconds\n", vxi->vxi_ageing);
@@ -274,7 +317,7 @@ static void vxlan_dump_details(struct rtnl_link *link, struct nl_dump_params *p)
 			nl_dump_line(p, "disabled\n");
 	}
 
-	if (vxi->vxi_mask & VXLAN_HAS_LIMIT) {
+	if (vxi->ce_mask & VXLAN_ATTR_LIMIT) {
 		nl_dump(p, "      limit ");
 		if (vxi->vxi_limit)
 			nl_dump_line(p, "%u\n", vxi->vxi_limit);
@@ -282,12 +325,12 @@ static void vxlan_dump_details(struct rtnl_link *link, struct nl_dump_params *p)
 			nl_dump_line(p, "unlimited\n");
 	}
 
-	if (vxi->vxi_mask & VXLAN_HAS_PORT_RANGE)
+	if (vxi->ce_mask & VXLAN_ATTR_PORT_RANGE)
 		nl_dump_line(p, "      port range %u - %u\n",
 					 ntohs(vxi->vxi_port_range.low),
 					 ntohs(vxi->vxi_port_range.high));
 
-	if (vxi->vxi_mask & VXLAN_HAS_PROXY) {
+	if (vxi->ce_mask & VXLAN_ATTR_PROXY) {
 		nl_dump(p, "      proxy ");
 		if (vxi->vxi_proxy)
 			nl_dump_line(p, "enabled (%#x)\n", vxi->vxi_proxy);
@@ -295,7 +338,7 @@ static void vxlan_dump_details(struct rtnl_link *link, struct nl_dump_params *p)
 			nl_dump_line(p, "disabled\n");
 	}
 
-	if (vxi->vxi_mask & VXLAN_HAS_RSC) {
+	if (vxi->ce_mask & VXLAN_ATTR_RSC) {
 		nl_dump(p, "      rsc ");
 		if (vxi->vxi_rsc)
 			nl_dump_line(p, "enabled (%#x)\n", vxi->vxi_rsc);
@@ -303,7 +346,7 @@ static void vxlan_dump_details(struct rtnl_link *link, struct nl_dump_params *p)
 			nl_dump_line(p, "disabled\n");
 	}
 
-	if (vxi->vxi_mask & VXLAN_HAS_L2MISS) {
+	if (vxi->ce_mask & VXLAN_ATTR_L2MISS) {
 		nl_dump(p, "      l2miss ");
 		if (vxi->vxi_l2miss)
 			nl_dump_line(p, "enabled (%#x)\n", vxi->vxi_l2miss);
@@ -311,7 +354,7 @@ static void vxlan_dump_details(struct rtnl_link *link, struct nl_dump_params *p)
 			nl_dump_line(p, "disabled\n");
 	}
 
-	if (vxi->vxi_mask & VXLAN_HAS_L3MISS) {
+	if (vxi->ce_mask & VXLAN_ATTR_L3MISS) {
 		nl_dump(p, "      l3miss ");
 		if (vxi->vxi_l3miss)
 			nl_dump_line(p, "enabled (%#x)\n", vxi->vxi_l3miss);
@@ -346,47 +389,53 @@ static int vxlan_put_attrs(struct nl_msg *msg, struct rtnl_link *link)
 	if (!(data = nla_nest_start(msg, IFLA_INFO_DATA)))
 		return -NLE_MSGSIZE;
 
-	if (vxi->vxi_mask & VXLAN_HAS_ID)
+	if (vxi->ce_mask & VXLAN_ATTR_ID)
 		NLA_PUT_U32(msg, IFLA_VXLAN_ID, vxi->vxi_id);
 
-	if (vxi->vxi_mask & VXLAN_HAS_GROUP)
+	if (vxi->ce_mask & VXLAN_ATTR_GROUP)
 		NLA_PUT(msg, IFLA_VXLAN_GROUP, sizeof(vxi->vxi_group), &vxi->vxi_group);
 
-	if (vxi->vxi_mask & VXLAN_HAS_LINK)
+	if (vxi->ce_mask & VXLAN_ATTR_GROUP6)
+		NLA_PUT(msg, IFLA_VXLAN_GROUP6, sizeof(vxi->vxi_group6), &vxi->vxi_group6);
+
+	if (vxi->ce_mask & VXLAN_ATTR_LINK)
 		NLA_PUT_U32(msg, IFLA_VXLAN_LINK, vxi->vxi_link);
 
-	if (vxi->vxi_mask & VXLAN_HAS_LOCAL)
+	if (vxi->ce_mask & VXLAN_ATTR_LOCAL)
 		NLA_PUT(msg, IFLA_VXLAN_LOCAL, sizeof(vxi->vxi_local), &vxi->vxi_local);
 
-	if (vxi->vxi_mask & VXLAN_HAS_TTL)
+	if (vxi->ce_mask & VXLAN_ATTR_LOCAL6)
+		NLA_PUT(msg, IFLA_VXLAN_LOCAL6, sizeof(vxi->vxi_local6), &vxi->vxi_local6);
+
+	if (vxi->ce_mask & VXLAN_ATTR_TTL)
 		NLA_PUT_U8(msg, IFLA_VXLAN_TTL, vxi->vxi_ttl);
 
-	if (vxi->vxi_mask & VXLAN_HAS_TOS)
+	if (vxi->ce_mask & VXLAN_ATTR_TOS)
 		NLA_PUT_U8(msg, IFLA_VXLAN_TOS, vxi->vxi_tos);
 
-	if (vxi->vxi_mask & VXLAN_HAS_LEARNING)
+	if (vxi->ce_mask & VXLAN_ATTR_LEARNING)
 		NLA_PUT_U8(msg, IFLA_VXLAN_LEARNING, vxi->vxi_learning);
 
-	if (vxi->vxi_mask & VXLAN_HAS_AGEING)
+	if (vxi->ce_mask & VXLAN_ATTR_AGEING)
 		NLA_PUT_U32(msg, IFLA_VXLAN_AGEING, vxi->vxi_ageing);
 
-	if (vxi->vxi_mask & VXLAN_HAS_LIMIT)
+	if (vxi->ce_mask & VXLAN_ATTR_LIMIT)
 		NLA_PUT_U32(msg, IFLA_VXLAN_LIMIT, vxi->vxi_limit);
 
-	if (vxi->vxi_mask & VXLAN_HAS_PORT_RANGE)
+	if (vxi->ce_mask & VXLAN_ATTR_PORT_RANGE)
 		NLA_PUT(msg, IFLA_VXLAN_PORT_RANGE, sizeof(vxi->vxi_port_range),
 				&vxi->vxi_port_range);
 
-	if (vxi->vxi_mask & VXLAN_HAS_PROXY)
+	if (vxi->ce_mask & VXLAN_ATTR_PROXY)
 		NLA_PUT_U8(msg, IFLA_VXLAN_PROXY, vxi->vxi_proxy);
 
-	if (vxi->vxi_mask & VXLAN_HAS_RSC)
+	if (vxi->ce_mask & VXLAN_ATTR_RSC)
 		NLA_PUT_U8(msg, IFLA_VXLAN_RSC, vxi->vxi_rsc);
 
-	if (vxi->vxi_mask & VXLAN_HAS_L2MISS)
+	if (vxi->ce_mask & VXLAN_ATTR_L2MISS)
 		NLA_PUT_U8(msg, IFLA_VXLAN_L2MISS, vxi->vxi_l2miss);
 
-	if (vxi->vxi_mask & VXLAN_HAS_L3MISS)
+	if (vxi->ce_mask & VXLAN_ATTR_L3MISS)
 		NLA_PUT_U8(msg, IFLA_VXLAN_L3MISS, vxi->vxi_l3miss);
 
 	nla_nest_end(msg, data);
@@ -396,6 +445,35 @@ nla_put_failure:
 	return 0;
 }
 
+static int vxlan_compare(struct rtnl_link *link_a, struct rtnl_link *link_b,
+			 int flags)
+{
+	struct vxlan_info *a = link_a->l_info;
+	struct vxlan_info *b = link_b->l_info;
+	int diff = 0;
+	uint32_t attrs = flags & LOOSE_COMPARISON ? b->ce_mask : ~0;
+
+#define VXLAN_DIFF(ATTR, EXPR) ATTR_DIFF(attrs, VXLAN_ATTR_##ATTR, a, b, EXPR)
+
+	diff |= VXLAN_DIFF(ID,    a->vxi_id    != b->vxi_id);
+	diff |= VXLAN_DIFF(GROUP, a->vxi_group != b->vxi_group);
+	diff |= VXLAN_DIFF(LINK,  a->vxi_link  != b->vxi_link);
+	diff |= VXLAN_DIFF(LOCAL, a->vxi_local != b->vxi_local);
+	diff |= VXLAN_DIFF(TOS,   a->vxi_tos   != b->vxi_tos);
+	diff |= VXLAN_DIFF(TTL,   a->vxi_ttl   != b->vxi_ttl);
+	diff |= VXLAN_DIFF(LEARNING, a->vxi_learning != b->vxi_learning);
+	diff |= VXLAN_DIFF(AGEING, a->vxi_ageing != b->vxi_ageing);
+	diff |= VXLAN_DIFF(PORT_RANGE,
+	                   a->vxi_port_range.low != b->vxi_port_range.low);
+	diff |= VXLAN_DIFF(PORT_RANGE,
+	                   a->vxi_port_range.high != b->vxi_port_range.high);
+	diff |= VXLAN_DIFF(GROUP6, memcmp(&a->vxi_group6, &b->vxi_group6, sizeof(a->vxi_group6)) != 0);
+	diff |= VXLAN_DIFF(LOCAL6,  memcmp(&a->vxi_local6, &b->vxi_local6, sizeof(a->vxi_local6)) != 0);
+#undef VXLAN_DIFF
+
+	return diff;
+}
+
 static struct rtnl_link_info_ops vxlan_info_ops = {
 	.io_name		= "vxlan",
 	.io_alloc		= vxlan_alloc,
@@ -407,6 +485,7 @@ static struct rtnl_link_info_ops vxlan_info_ops = {
 	.io_clone		= vxlan_clone,
 	.io_put_attrs		= vxlan_put_attrs,
 	.io_free		= vxlan_free,
+	.io_compare             = vxlan_compare,
 };
 
 /** @cond SKIP */
@@ -471,7 +550,7 @@ int rtnl_link_vxlan_set_id(struct rtnl_link *link, uint32_t id)
 		return -NLE_INVAL;
 
 	vxi->vxi_id = id;
-	vxi->vxi_mask |= VXLAN_HAS_ID;
+	vxi->ce_mask |= VXLAN_ATTR_ID;
 
 	return 0;
 }
@@ -492,7 +571,7 @@ int rtnl_link_vxlan_get_id(struct rtnl_link *link, uint32_t *id)
 	if(!id)
 		return -NLE_INVAL;
 
-	if (vxi->vxi_mask & VXLAN_HAS_ID)
+	if (vxi->ce_mask & VXLAN_ATTR_ID)
 		*id = vxi->vxi_id;
 	else
 		return -NLE_AGAIN;
@@ -513,14 +592,21 @@ int rtnl_link_vxlan_set_group(struct rtnl_link *link, struct nl_addr *addr)
 
 	IS_VXLAN_LINK_ASSERT(link);
 
-	if ((nl_addr_get_family(addr) != AF_INET) ||
-		(nl_addr_get_len(addr) != sizeof(vxi->vxi_group)))
+	if ((nl_addr_get_family(addr) == AF_INET) &&
+	    (nl_addr_get_len(addr) == sizeof(vxi->vxi_group))) {
+		memcpy(&vxi->vxi_group, nl_addr_get_binary_addr(addr),
+		       sizeof(vxi->vxi_group));
+		vxi->ce_mask |= VXLAN_ATTR_GROUP;
+		vxi->ce_mask &= ~VXLAN_ATTR_GROUP6;
+	} else if ((nl_addr_get_family(addr) == AF_INET6) &&
+		   (nl_addr_get_len(addr) == sizeof(vxi->vxi_group6))) {
+		memcpy(&vxi->vxi_group6, nl_addr_get_binary_addr(addr),
+		       sizeof(vxi->vxi_group6));
+		vxi->ce_mask |= VXLAN_ATTR_GROUP6;
+		vxi->ce_mask &= ~VXLAN_ATTR_GROUP;
+	} else
 		return -NLE_INVAL;
 
-	memcpy(&vxi->vxi_group, nl_addr_get_binary_addr(addr),
-		   sizeof(vxi->vxi_group));
-	vxi->vxi_mask |= VXLAN_HAS_GROUP;
-
 	return 0;
 }
 
@@ -540,11 +626,13 @@ int rtnl_link_vxlan_get_group(struct rtnl_link *link, struct nl_addr **addr)
 	if (!addr)
 		return -NLE_INVAL;
 
-	if (!(vxi->vxi_mask & VXLAN_HAS_GROUP))
+	if (vxi->ce_mask & VXLAN_ATTR_GROUP)
+		*addr = nl_addr_build(AF_INET, &vxi->vxi_group, sizeof(vxi->vxi_group));
+	else if (vxi->ce_mask & VXLAN_ATTR_GROUP6)
+		*addr = nl_addr_build(AF_INET6, &vxi->vxi_group6, sizeof(vxi->vxi_group6));
+	else
 		return -NLE_AGAIN;
 
-	*addr = nl_addr_build(AF_INET, &vxi->vxi_group, sizeof(vxi->vxi_group));
-
 	return 0;
 }
 
@@ -562,7 +650,7 @@ int rtnl_link_vxlan_set_link(struct rtnl_link *link, uint32_t index)
 	IS_VXLAN_LINK_ASSERT(link);
 
 	vxi->vxi_link = index;
-	vxi->vxi_mask |= VXLAN_HAS_LINK;
+	vxi->ce_mask |= VXLAN_ATTR_LINK;
 
 	return 0;
 }
@@ -583,7 +671,7 @@ int rtnl_link_vxlan_get_link(struct rtnl_link *link, uint32_t *index)
 	if (!index)
 		return -NLE_INVAL;
 
-	if (!(vxi->vxi_mask & VXLAN_HAS_LINK))
+	if (!(vxi->ce_mask & VXLAN_ATTR_LINK))
 		return -NLE_AGAIN;
 
 	*index = vxi->vxi_link;
@@ -604,14 +692,21 @@ int rtnl_link_vxlan_set_local(struct rtnl_link *link, struct nl_addr *addr)
 
 	IS_VXLAN_LINK_ASSERT(link);
 
-	if ((nl_addr_get_family(addr) != AF_INET) ||
-		(nl_addr_get_len(addr) != sizeof(vxi->vxi_local)))
+	if ((nl_addr_get_family(addr) == AF_INET) &&
+	    (nl_addr_get_len(addr) == sizeof(vxi->vxi_local))) {
+		memcpy(&vxi->vxi_local, nl_addr_get_binary_addr(addr),
+		       sizeof(vxi->vxi_local));
+		vxi->ce_mask |= VXLAN_ATTR_LOCAL;
+		vxi->ce_mask &= VXLAN_ATTR_LOCAL6;
+	} else if ((nl_addr_get_family(addr) == AF_INET6) &&
+		   (nl_addr_get_len(addr) == sizeof(vxi->vxi_local6))) {
+		memcpy(&vxi->vxi_local6, nl_addr_get_binary_addr(addr),
+		       sizeof(vxi->vxi_local6));
+		vxi->ce_mask |= VXLAN_ATTR_LOCAL6;
+		vxi->ce_mask &= ~VXLAN_ATTR_LOCAL;
+	} else
 		return -NLE_INVAL;
 
-	memcpy(&vxi->vxi_local, nl_addr_get_binary_addr(addr),
-		   sizeof(vxi->vxi_local));
-	vxi->vxi_mask |= VXLAN_HAS_LOCAL;
-
 	return 0;
 }
 
@@ -631,11 +726,13 @@ int rtnl_link_vxlan_get_local(struct rtnl_link *link, struct nl_addr **addr)
 	if (!addr)
 		return -NLE_INVAL;
 
-	if (!(vxi->vxi_mask & VXLAN_HAS_LOCAL))
+	if (vxi->ce_mask & VXLAN_ATTR_LOCAL)
+		*addr = nl_addr_build(AF_INET, &vxi->vxi_local, sizeof(vxi->vxi_local));
+	else if (vxi->ce_mask & VXLAN_ATTR_LOCAL6)
+		*addr = nl_addr_build(AF_INET6, &vxi->vxi_local6, sizeof(vxi->vxi_local6));
+	else
 		return -NLE_AGAIN;
 
-	*addr = nl_addr_build(AF_INET, &vxi->vxi_local, sizeof(vxi->vxi_local));
-
 	return 0;
 }
 
@@ -653,7 +750,7 @@ int rtnl_link_vxlan_set_ttl(struct rtnl_link *link, uint8_t ttl)
 	IS_VXLAN_LINK_ASSERT(link);
 
 	vxi->vxi_ttl = ttl;
-	vxi->vxi_mask |= VXLAN_HAS_TTL;
+	vxi->ce_mask |= VXLAN_ATTR_TTL;
 
 	return 0;
 }
@@ -670,7 +767,7 @@ int rtnl_link_vxlan_get_ttl(struct rtnl_link *link)
 
 	IS_VXLAN_LINK_ASSERT(link);
 
-	if (!(vxi->vxi_mask & VXLAN_HAS_TTL))
+	if (!(vxi->ce_mask & VXLAN_ATTR_TTL))
 		return -NLE_AGAIN;
 
 	return vxi->vxi_ttl;
@@ -690,7 +787,7 @@ int rtnl_link_vxlan_set_tos(struct rtnl_link *link, uint8_t tos)
 	IS_VXLAN_LINK_ASSERT(link);
 
 	vxi->vxi_tos = tos;
-	vxi->vxi_mask |= VXLAN_HAS_TOS;
+	vxi->ce_mask |= VXLAN_ATTR_TOS;
 
 	return 0;
 }
@@ -707,7 +804,7 @@ int rtnl_link_vxlan_get_tos(struct rtnl_link *link)
 
 	IS_VXLAN_LINK_ASSERT(link);
 
-	if (!(vxi->vxi_mask & VXLAN_HAS_TOS))
+	if (!(vxi->ce_mask & VXLAN_ATTR_TOS))
 		return -NLE_AGAIN;
 
 	return vxi->vxi_tos;
@@ -727,7 +824,7 @@ int rtnl_link_vxlan_set_learning(struct rtnl_link *link, uint8_t learning)
 	IS_VXLAN_LINK_ASSERT(link);
 
 	vxi->vxi_learning = learning;
-	vxi->vxi_mask |= VXLAN_HAS_LEARNING;
+	vxi->ce_mask |= VXLAN_ATTR_LEARNING;
 
 	return 0;
 }
@@ -744,7 +841,7 @@ int rtnl_link_vxlan_get_learning(struct rtnl_link *link)
 
 	IS_VXLAN_LINK_ASSERT(link);
 
-	if (!(vxi->vxi_mask & VXLAN_HAS_LEARNING))
+	if (!(vxi->ce_mask & VXLAN_ATTR_LEARNING))
 		return -NLE_AGAIN;
 
 	return vxi->vxi_learning;
@@ -786,7 +883,7 @@ int rtnl_link_vxlan_set_ageing(struct rtnl_link *link, uint32_t expiry)
 	IS_VXLAN_LINK_ASSERT(link);
 
 	vxi->vxi_ageing = expiry;
-	vxi->vxi_mask |= VXLAN_HAS_AGEING;
+	vxi->ce_mask |= VXLAN_ATTR_AGEING;
 
 	return 0;
 }
@@ -807,7 +904,7 @@ int rtnl_link_vxlan_get_ageing(struct rtnl_link *link, uint32_t *expiry)
 	if (!expiry)
 		return -NLE_INVAL;
 
-	if (vxi->vxi_mask & VXLAN_HAS_AGEING)
+	if (vxi->ce_mask & VXLAN_ATTR_AGEING)
 		*expiry = vxi->vxi_ageing;
 	else
 		return -NLE_AGAIN;
@@ -829,7 +926,7 @@ int rtnl_link_vxlan_set_limit(struct rtnl_link *link, uint32_t limit)
 	IS_VXLAN_LINK_ASSERT(link);
 
 	vxi->vxi_limit = limit;
-	vxi->vxi_mask |= VXLAN_HAS_LIMIT;
+	vxi->ce_mask |= VXLAN_ATTR_LIMIT;
 
 	return 0;
 }
@@ -850,7 +947,7 @@ int rtnl_link_vxlan_get_limit(struct rtnl_link *link, uint32_t *limit)
 	if (!limit)
 		return -NLE_INVAL;
 
-	if (vxi->vxi_mask & VXLAN_HAS_LIMIT)
+	if (vxi->ce_mask & VXLAN_ATTR_LIMIT)
 		*limit = vxi->vxi_limit;
 	else
 		return -NLE_AGAIN;
@@ -876,7 +973,7 @@ int rtnl_link_vxlan_set_port_range(struct rtnl_link *link,
 		return -NLE_INVAL;
 
 	memcpy(&vxi->vxi_port_range, range, sizeof(vxi->vxi_port_range));
-	vxi->vxi_mask |= VXLAN_HAS_PORT_RANGE;
+	vxi->ce_mask |= VXLAN_ATTR_PORT_RANGE;
 
 	return 0;
 }
@@ -898,7 +995,7 @@ int rtnl_link_vxlan_get_port_range(struct rtnl_link *link,
 	if (!range)
 		return -NLE_INVAL;
 
-	if (vxi->vxi_mask & VXLAN_HAS_PORT_RANGE)
+	if (vxi->ce_mask & VXLAN_ATTR_PORT_RANGE)
 		memcpy(range, &vxi->vxi_port_range, sizeof(*range));
 	else
 		return -NLE_AGAIN;
@@ -920,7 +1017,7 @@ int rtnl_link_vxlan_set_proxy(struct rtnl_link *link, uint8_t proxy)
 	IS_VXLAN_LINK_ASSERT(link);
 
 	vxi->vxi_proxy = proxy;
-	vxi->vxi_mask |= VXLAN_HAS_PROXY;
+	vxi->ce_mask |= VXLAN_ATTR_PROXY;
 
 	return 0;
 }
@@ -937,7 +1034,7 @@ int rtnl_link_vxlan_get_proxy(struct rtnl_link *link)
 
 	IS_VXLAN_LINK_ASSERT(link);
 
-	if (!(vxi->vxi_mask & VXLAN_HAS_PROXY))
+	if (!(vxi->ce_mask & VXLAN_ATTR_PROXY))
 		return -NLE_AGAIN;
 
 	return vxi->vxi_proxy;
@@ -979,7 +1076,7 @@ int rtnl_link_vxlan_set_rsc(struct rtnl_link *link, uint8_t rsc)
 	IS_VXLAN_LINK_ASSERT(link);
 
 	vxi->vxi_rsc = rsc;
-	vxi->vxi_mask |= VXLAN_HAS_RSC;
+	vxi->ce_mask |= VXLAN_ATTR_RSC;
 
 	return 0;
 }
@@ -996,7 +1093,7 @@ int rtnl_link_vxlan_get_rsc(struct rtnl_link *link)
 
 	IS_VXLAN_LINK_ASSERT(link);
 
-	if (!(vxi->vxi_mask & VXLAN_HAS_RSC))
+	if (!(vxi->ce_mask & VXLAN_ATTR_RSC))
 		return -NLE_AGAIN;
 
 	return vxi->vxi_rsc;
@@ -1038,7 +1135,7 @@ int rtnl_link_vxlan_set_l2miss(struct rtnl_link *link, uint8_t miss)
 	IS_VXLAN_LINK_ASSERT(link);
 
 	vxi->vxi_l2miss = miss;
-	vxi->vxi_mask |= VXLAN_HAS_L2MISS;
+	vxi->ce_mask |= VXLAN_ATTR_L2MISS;
 
 	return 0;
 }
@@ -1055,7 +1152,7 @@ int rtnl_link_vxlan_get_l2miss(struct rtnl_link *link)
 
 	IS_VXLAN_LINK_ASSERT(link);
 
-	if (!(vxi->vxi_mask & VXLAN_HAS_L2MISS))
+	if (!(vxi->ce_mask & VXLAN_ATTR_L2MISS))
 		return -NLE_AGAIN;
 
 	return vxi->vxi_l2miss;
@@ -1097,7 +1194,7 @@ int rtnl_link_vxlan_set_l3miss(struct rtnl_link *link, uint8_t miss)
 	IS_VXLAN_LINK_ASSERT(link);
 
 	vxi->vxi_l3miss = miss;
-	vxi->vxi_mask |= VXLAN_HAS_L3MISS;
+	vxi->ce_mask |= VXLAN_ATTR_L3MISS;
 
 	return 0;
 }
@@ -1114,7 +1211,7 @@ int rtnl_link_vxlan_get_l3miss(struct rtnl_link *link)
 
 	IS_VXLAN_LINK_ASSERT(link);
 
-	if (!(vxi->vxi_mask & VXLAN_HAS_L3MISS))
+	if (!(vxi->ce_mask & VXLAN_ATTR_L3MISS))
 		return -NLE_AGAIN;
 
 	return vxi->vxi_l3miss;
